﻿using System;
using System.Collections.Generic;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Communication
{
    public class SerialPort_Function//串口配置类
    {
        public delegate void Close_Port();                  //热拔插动作委托        
        public static event Close_Port Event_Colse_Port;    //热拔插动作事件

        /*****************************************************************************************************************
        系统控件定义
        ******************************************************************************************************************/
        public static SerialPort eCoder_SerialPort = new SerialPort();//实例化一个SerialPort
        private static System.Timers.Timer Timer_Communcation_Timeout = new System.Timers.Timer();//实例化一个定时器

        /*****************************************************************************************************************
        事件标记和计数变量
        ******************************************************************************************************************/
        private static int int_timeout = 0;
        private static bool bool_Receive_Event_Finish_Flag = false;
        private static bool bool_Communication_TimeOut_Flag = false;

        /*****************************************************************************************************************
        串口配置参数
        ******************************************************************************************************************/
        public enum enum_Rec_Status { No_Connect = 0, No_Reply = 1, Reply_Err = 2, CRC_Err = 3, Correct = 4 };//未连接0 无回复1，错误回复2，CRC错误3，正确回复4
        public struct Rec_Buffer
        {
            public byte[] Buffer;//接收缓存数据
            public enum_Rec_Status Rec_Status;
        }
        public static Rec_Buffer Struct_Rec_Buffer = new Rec_Buffer();
 
        public struct Port_Config_Struct
        {
            public string port_Name;//串口名称
            public string[] port_Name_Num;//串口名称
            public int port_BaudRate;//串口波特率
            public int port_DataBits;//字节标准长度
            public Parity port_Parity;//奇偶校验位
            public StopBits port_StopBits;//停止位
            public int port_ReadBufferSize;//接收缓存大小
            public int port_TimeOut;//接收超时时间自定义
        }
        public static Port_Config_Struct Struct_Port_Config = new Port_Config_Struct();//实例化串口配置结构

        public static void Struct_Port_Config_Init(int BaudRate,string PortName)//配置初始化
        {
            Struct_Port_Config.port_Name = PortName;//配置串口名称
            Struct_Port_Config.port_BaudRate = BaudRate;//配置串口波特率
            Struct_Port_Config.port_Parity = Parity.None;//奇偶校验无
            Struct_Port_Config.port_DataBits = 8;//字节标准长度8
            Struct_Port_Config.port_StopBits = StopBits.One;//停止位1
            Struct_Port_Config.port_ReadBufferSize = 300000;//接收缓存大小
            Struct_Port_Config.port_TimeOut = 500;//n*ms超时
        }

        /*****************************************************************************************************************
        函数名：Config_Port
        输入值：无
        返回值：无   
        功能说明：对串口的参数进行配置（串口号、串口波特率、串口读取缓存区间大小、串口通信数据宽度、停止位、奇偶校验位配置）
        ******************************************************************************************************************/
        public static void Config_Port()//配置串口信息
        {
            eCoder_SerialPort.PortName = Struct_Port_Config.port_Name;
            eCoder_SerialPort.BaudRate = Struct_Port_Config.port_BaudRate;
            eCoder_SerialPort.Parity = Struct_Port_Config.port_Parity;
            eCoder_SerialPort.DataBits = Struct_Port_Config.port_DataBits;
            eCoder_SerialPort.StopBits = Struct_Port_Config.port_StopBits;
            eCoder_SerialPort.ReadBufferSize = Struct_Port_Config.port_ReadBufferSize;
        }

        /*****************************************************************************************************************
        函数名：Open_SerialPort
        输入值：无
        返回值：int
        功能说明：打开串口并返回串口打开结果，打开成功则返回1,失败则返回0
        ******************************************************************************************************************/
        public static int Open_SerialPort()//打开串口
        {
            Config_Port();
            try
            {
                eCoder_SerialPort.Open();
                return 1;
            }
            catch
            {
                return 0;
            }
        }

        /*****************************************************************************************************************
        函数名：Close_SerialPort
        输入值：无
        返回值：无
        功能说明：关闭串口
        ******************************************************************************************************************/
        public static void Close_SerialPort()//关闭串口
        {
            eCoder_SerialPort.Close();
        }

        /*****************************************************************************************************************
        函数名：PortName_Config
        输入值：系统控件ComboBox
        返回值：无
        功能说明：清除当前串口号→重新扫描当前存在串口号→尝试打开所有扫描到的串口→若打开成功，则添加至下拉框中
        ******************************************************************************************************************/
        public static void PortName_Config(ComboBox cbx)//串口号配置
        {
            cbx.Items.Clear();//清除当前串口号
            Struct_Port_Config.port_Name_Num = SerialPort.GetPortNames();//扫描当前存在串口号
            if (Struct_Port_Config.port_Name_Num.Length > 0)
            {
                for (int i = 0; i < Struct_Port_Config.port_Name_Num.Length; i++)
                {
                    try
                    {
                        eCoder_SerialPort.PortName = Struct_Port_Config.port_Name_Num[i];
                        eCoder_SerialPort.Open();
                        eCoder_SerialPort.Close();
                        cbx.Items.Add(Struct_Port_Config.port_Name_Num[i]);
                    }
                    catch { }
                }
                cbx.SelectedIndex = cbx.Items.Count > 0 ? 0 : -1;//设置下拉框索引
            }
            else
            {
                cbx.Text = string.Empty;//若当前无串口号存在，则为空
            }
        }

        /*****************************************************************************************************************
        函数名：BaudRate_Config
        输入值：系统控件ComboBox
        返回值：无
        功能说明：配置串口波特率
        ******************************************************************************************************************/
        public static void BaudRate_Config(ComboBox cbx)//波特率配置
        {
            cbx.Items.Add("2500000");//2.5M波特率
            cbx.SelectedIndex = 0;//设置下拉框索引
        }

        /*****************************************************************************************************************
        函数名：Send_Data
        输入值：byte[] 发送的数据，RecData_Length返回数据的长度
        返回值：byte[]串口所接收数据
        功能说明：串口发送数据，并返回接收到的数据
        ******************************************************************************************************************/
        private static bool bool_Timer_Lock = false;//防止再次新建定时器导致崩溃
        public static byte[] Send_Data(byte[] Data, int RecData_Length)
        {
            Struct_Rec_Buffer.Rec_Status = enum_Rec_Status.No_Connect;
            Algorithm.CRC_Check CRC = new Algorithm.CRC_Check();
            try
            {
                if (!eCoder_SerialPort.IsOpen)//防止热拔插
                {
                    Event_Colse_Port();
                    return Struct_Rec_Buffer.Buffer;
                }
                Clear_Port_Buffer();
                eCoder_SerialPort.Write(Data, 0, Data.Length);
                if (!bool_Timer_Lock)
                {
                    bool_Timer_Lock = true;
                    Timer_Communcation_Timeout.Elapsed += new System.Timers.ElapsedEventHandler(TimeOut_Tick);
                }
                Timer_Communcation_Timeout.Interval = 1;
                Timer_Communcation_Timeout.Enabled = true;
                Timer_Communcation_Timeout.Start();
                bool_Communication_TimeOut_Flag = false;
                bool_Receive_Event_Finish_Flag = false;
                if (!eCoder_SerialPort.IsOpen)//防止热拔插
                {
                    Event_Colse_Port();
                    return Struct_Rec_Buffer.Buffer;
                }
                while (!bool_Receive_Event_Finish_Flag && !bool_Communication_TimeOut_Flag)
                {
                    if (eCoder_SerialPort.BytesToRead == RecData_Length)
                    {
                        Timer_Communcation_Timeout.Stop();
                        bool_Receive_Event_Finish_Flag = true;
                        Struct_Rec_Buffer.Buffer = new byte[eCoder_SerialPort.BytesToRead];
                        eCoder_SerialPort.Read(Struct_Rec_Buffer.Buffer, 0, Struct_Rec_Buffer.Buffer.Length);
                        if (Struct_Rec_Buffer.Buffer[Struct_Rec_Buffer.Buffer.Length - 1] == CRC.CRC8Calculate(Struct_Rec_Buffer.Buffer, Struct_Rec_Buffer.Buffer.Length - 1))
                        {
                            Struct_Rec_Buffer.Rec_Status = enum_Rec_Status.Correct;
                        }
                        else
                        {
                            Struct_Rec_Buffer.Rec_Status = enum_Rec_Status.CRC_Err;
                        }
                    }
                }
                if (bool_Communication_TimeOut_Flag)//若触发计数超时
                {
                    Struct_Rec_Buffer.Buffer = new byte[eCoder_SerialPort.BytesToRead];
                    Clear_Recvied_Buffer();
                    if (Struct_Rec_Buffer.Buffer.Length == 0)
                    {
                        Struct_Rec_Buffer.Rec_Status = enum_Rec_Status.No_Reply;
                    }
                    else
                    {
                        Struct_Rec_Buffer.Rec_Status = enum_Rec_Status.Reply_Err;
                    }
                }
            }
            catch { }
            int_timeout = 0;//计数初始化
            Timer_Communcation_Timeout.Stop();//关闭定时器
            return Struct_Rec_Buffer.Buffer;
        }

        /*****************************************************************************************************************
        函数名：TimeOut_Tick
        功能说明：定时器触发事件
        逻辑说明：每次进入cnt_timeout++，当大于port_TimeOut时触发清除接收缓存，并置位接收超时标记
        ******************************************************************************************************************/
        private static void TimeOut_Tick(object sender, EventArgs e)//接收超时计时
        {
            int_timeout++;
            if (int_timeout > Struct_Port_Config.port_TimeOut)
            {
                bool_Communication_TimeOut_Flag = true;
                Timer_Communcation_Timeout.Stop();
            }
        }

        /*****************************************************************************************************************
        函数名：Clear_Port_Buffer
        输入值：无
        返回值：无
        功能说明：清除串口缓存
        ******************************************************************************************************************/
        private static void Clear_Port_Buffer()
        {
            byte[] buffer = new byte[eCoder_SerialPort.BytesToRead];
            eCoder_SerialPort.Read(buffer, 0, buffer.Length);
        }

        /*****************************************************************************************************************
        函数名：Clear_Recived_Buffer
        输入值：无
        返回值：无
        功能说明：清除接收缓存
        ******************************************************************************************************************/
        private static void Clear_Recvied_Buffer()
        {
            for (int i = 0; i < Struct_Rec_Buffer.Buffer.Length; i++)
            {
                Struct_Rec_Buffer.Buffer[i] = 0;
            }
        }
    }
}
