﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Communication
{   
    public class ZeroErr_Communication
    {
        public delegate void Print_Message(string text);                //更新信息委托
        public static event Print_Message Event_Print_Message;          //更新信息事件

        public static byte[] eCoder_Information_Data = new byte[11];    //位置、分辨率读取指令原始数据
        public static byte[] eCoder_Single_Turn = new byte[6];          //单圈读取指令原始数据

        public struct Rec_Buffer//接收缓存
        {
            public byte[] Buffer;
        }
        public static Rec_Buffer Struct_Rec_Buffer = new Rec_Buffer();

        public struct Request_Mode//编码器通讯请求
        {
            public bool read_eCoder_Information;    //读取编码器信息 单多圈位置、分辨率
            public bool read_eCoder_Status;         //读取编码器状态信息
            public bool set_Single_Turn_To_Zero;    //编码器单圈清零
            public bool set_Multi_Turn_To_Zero;     //编码器多圈复位
        }
        public static Request_Mode Struct_Request_Mode = new Request_Mode();
        public static void Struct_Request_Mode_Init()//编码器通讯请求初始化
        {
            Struct_Request_Mode.read_eCoder_Information = false;
            Struct_Request_Mode.read_eCoder_Status = false;
            Struct_Request_Mode.set_Single_Turn_To_Zero = false;
            Struct_Request_Mode.set_Multi_Turn_To_Zero = false;
        }

        public struct eCoder_Data//编码器位置反馈数据
        {
            public int single_Turn;     //单圈值
            public int multi_Turn;      //多圈值
            public int resolution;      //分辨率
            public double Angle;        //角度值
        }
        public static eCoder_Data Struct_eCoder_Data = new eCoder_Data();
        public static void Struct_eCoder_Data_Init()//编码器位置反馈数据初始化
        {
            Struct_eCoder_Data.single_Turn = 0;
            Struct_eCoder_Data.multi_Turn = 0;
            Struct_eCoder_Data.resolution = 0;
            Struct_eCoder_Data.Angle = 0;
        }

        public struct eCoder_Inside_Status//编码器内部状态
        {
            public byte eCoder_SA;          //编码器状态位
            public byte eCoder_ALMC;        //编码器错误信息状态
            public string eCoder_Status;    //编码器状态
        }
        public static eCoder_Inside_Status Struct_eCoder_Inside_Status = new eCoder_Inside_Status();
        public static void Struct_eCoder_Inside_Status_Init()//编码器内部状态初始化
        {
            Struct_eCoder_Inside_Status.eCoder_SA = 0;
            Struct_eCoder_Inside_Status.eCoder_ALMC = 0;
            Struct_eCoder_Inside_Status.eCoder_Status = string.Empty;
        }

        /*****************************************************************************************************************
        函数名：Read_eCoder_Information
        输入值：无
        返回值：读取正确返回1，读取错误返回0   
        功能说明：读取编码器信息 单圈多圈位置、分辨率
        ******************************************************************************************************************/
        public static int Read_eCoder_Information()
        {
            Struct_Rec_Buffer.Buffer = SerialPort_Function.Send_Data(ZeroErr_Command.Cmd_Read_All, 11);//开始通讯
            if (SerialPort_Function.Struct_Rec_Buffer.Rec_Status == SerialPort_Function.enum_Rec_Status.Correct)//判断数据是否正确
            {
                if (Struct_Rec_Buffer.Buffer.Length == 11)//判断数据长度是否正确
                {
                    eCoder_Information_Data = Struct_Rec_Buffer.Buffer;//原始数据提取
                    Data_Conversion.Information_Data_Conversion();//原始数据转换
                    Data_Conversion.eCoder_Angle_Conversion();//角度数据转换
                    Data_Conversion.eCoder_Inside_Status_Conversion();//原始状态数据转换
                }
                else//字节长度错误
                {
                    Event_Print_Message("字节长度错误\r\n");
                }
            }
            else if (SerialPort_Function.Struct_Rec_Buffer.Rec_Status == SerialPort_Function.enum_Rec_Status.CRC_Err)//CRC错误
            {
                Event_Print_Message("通讯数据CRC错误\r\n");
                return 0;
            }
            else if (SerialPort_Function.Struct_Rec_Buffer.Rec_Status == SerialPort_Function.enum_Rec_Status.No_Reply)//无回复
            {
                Event_Print_Message("通讯数据无回复\r\n");
                return 0;
            }
            else if (SerialPort_Function.Struct_Rec_Buffer.Rec_Status == SerialPort_Function.enum_Rec_Status.Reply_Err)//错误回复
            {
                Event_Print_Message("通讯数据回复错误\r\n");
                return 0;
            }
            else if (SerialPort_Function.Struct_Rec_Buffer.Rec_Status == SerialPort_Function.enum_Rec_Status.No_Connect)//未连接
            {
                return 0;
            }
            return 1;
        }

        /*****************************************************************************************************************
        函数名：Set_Single_To_Zero
        输入值：无
        返回值：读取正确返回1，读取错误返回0   
        功能说明：单圈清零
        ******************************************************************************************************************/
        public static int Set_Single_To_Zero()
        {
            for (int i = 0; i < 10; i++)//指令下发10次
            {
                Struct_Rec_Buffer.Buffer = SerialPort_Function.Send_Data(ZeroErr_Command.Cmd_Set_Single_To_Zero, 6);//开始通讯
                if (SerialPort_Function.Struct_Rec_Buffer.Rec_Status == SerialPort_Function.enum_Rec_Status.Correct)//判断数据是否正确
                {
                    if (Struct_Rec_Buffer.Buffer.Length == 6)//判断字节长度是否正确
                    {
                        eCoder_Single_Turn = Struct_Rec_Buffer.Buffer;//原始数据提取
                        Data_Conversion.Single_Turn_Conversion();//原始数据转换
                    }
                    else//字节长度错误
                    {
                        Event_Print_Message("字节长度错误\r\n");
                    }
                }
                else if (SerialPort_Function.Struct_Rec_Buffer.Rec_Status == SerialPort_Function.enum_Rec_Status.CRC_Err)//CRC错误
                {
                    Event_Print_Message("通讯数据CRC错误\r\n");
                    return 0;
                }
                else if (SerialPort_Function.Struct_Rec_Buffer.Rec_Status == SerialPort_Function.enum_Rec_Status.No_Reply)//无回复
                {
                    Event_Print_Message("通讯数据无回复\r\n");
                    return 0;
                }
                else if (SerialPort_Function.Struct_Rec_Buffer.Rec_Status == SerialPort_Function.enum_Rec_Status.Reply_Err)//错误回复
                {
                    Event_Print_Message("通讯数据回复错误\r\n");
                    return 0;
                }
                else if (SerialPort_Function.Struct_Rec_Buffer.Rec_Status == SerialPort_Function.enum_Rec_Status.No_Connect)//未连接
                {
                    return 0;
                }
            }
            return 1;
        }

        /*****************************************************************************************************************
        函数名：Set_Multi_To_Zero
        输入值：无
        返回值：读取正确返回1，读取错误返回0   
        功能说明：多圈复位
        ******************************************************************************************************************/
        public static int Set_Multi_To_Zero()
        {
            for (int i = 0; i < 10; i++)//指令下发10次
            {
                Struct_Rec_Buffer.Buffer = SerialPort_Function.Send_Data(ZeroErr_Command.Cmd_Set_Multi_To_Zero, 6);//开始通讯
                if (SerialPort_Function.Struct_Rec_Buffer.Rec_Status == SerialPort_Function.enum_Rec_Status.Correct)//判断数据是否正确
                {
                    if (Struct_Rec_Buffer.Buffer.Length == 6)//判断字节长度是否正确
                    {
                        eCoder_Single_Turn = Struct_Rec_Buffer.Buffer;//原始数据提取
                        Data_Conversion.Single_Turn_Conversion();//原始数据转换
                    }
                    else//字节长度错误
                    {
                        Event_Print_Message("字节长度错误\r\n");
                    }
                }
                else if (SerialPort_Function.Struct_Rec_Buffer.Rec_Status == SerialPort_Function.enum_Rec_Status.CRC_Err)//CRC错误
                {
                    Event_Print_Message("通讯数据CRC错误\r\n");
                    return 0;
                }
                else if (SerialPort_Function.Struct_Rec_Buffer.Rec_Status == SerialPort_Function.enum_Rec_Status.No_Reply)//无回复
                {
                    Event_Print_Message("通讯数据无回复\r\n");
                    return 0;
                }
                else if (SerialPort_Function.Struct_Rec_Buffer.Rec_Status == SerialPort_Function.enum_Rec_Status.Reply_Err)//错误回复
                {
                    Event_Print_Message("通讯数据回复错误\r\n");
                    return 0;
                }
                else if (SerialPort_Function.Struct_Rec_Buffer.Rec_Status == SerialPort_Function.enum_Rec_Status.No_Connect)//未连接
                {
                    return 0;
                }
            }
            return 1;
        }

        /*****************************************************************************************************************
        函数名：Read_eCoder_Inside_Status
        输入值：无
        返回值：无  
        功能说明：读取编码器内部状态
        ******************************************************************************************************************/
        public static void Read_eCoder_Inside_Status()
        {
            if (SerialPort_Function.Struct_Rec_Buffer.Rec_Status == SerialPort_Function.enum_Rec_Status.Correct)
            {
                Event_Print_Message(ZeroErr_Communication.Struct_eCoder_Inside_Status.eCoder_Status);
            }
        }
    }
}
