﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Communication;
using System.Threading;

namespace eCoder_UI
{
    public partial class MainUI : Form
    {
        private Task CurrentTask;
        private bool Task_Status = false;
        private ManualResetEvent ResetEvent = new ManualResetEvent(true);
        private System.Windows.Forms.Timer timer = new System.Windows.Forms.Timer();
        

        public MainUI()//构造函数
        {
            InitializeComponent();//资源初始化
        }

        private void MainUI_Load(object sender, EventArgs e)//窗体加载函数
        {
            timer.Interval = 100;
            timer.Tick += new System.EventHandler(this.timer_Tick);
            timer.Start();
            SerialPort_Function.PortName_Config(cbx_PortName);//串口号配置
            SerialPort_Function.BaudRate_Config(cbx_BaudRate);//波特率配置
            Communication.ZeroErr_Communication.Event_Print_Message += new ZeroErr_Communication.Print_Message(Print_Message);//订阅更新信息事件
            Communication.SerialPort_Function.Event_Colse_Port += new SerialPort_Function.Close_Port(Close_Port);
        }

        private void timer_Tick(object sender, EventArgs e)//定时器刷新
        {
            tbx_Single_Turn.Text = Communication.ZeroErr_Communication.Struct_eCoder_Data.single_Turn.ToString();
            tbx_Multi_Turn.Text = Communication.ZeroErr_Communication.Struct_eCoder_Data.multi_Turn.ToString();
            tbx_Res.Text = Communication.ZeroErr_Communication.Struct_eCoder_Data.resolution.ToString();
            tbx_Angle.Text = Communication.ZeroErr_Communication.Struct_eCoder_Data.Angle.ToString();
        }

        private void btn_Scan_Click(object sender, EventArgs e)//刷新按钮点击事件
        {
            SerialPort_Function.PortName_Config(cbx_PortName);//串口号配置
        }

        private void btn_Connect_Click(object sender, EventArgs e)//连接按钮点击事件
        {
            if(btn_Connect.Text == "连接" && cbx_PortName.Text != string.Empty && cbx_BaudRate.Text != string.Empty)
            {
                SerialPort_Function.Struct_Port_Config_Init(Convert.ToInt32(cbx_BaudRate.Text), cbx_PortName.Text);//串口配置初始化
                if (SerialPort_Function.Open_SerialPort() == 1)//若串口打开成功
                {
                    Communication.Communication_Function.err_count = 0;//错误计数初始化
                    Communication.ZeroErr_Communication.Struct_Request_Mode.read_eCoder_Information = true;//读取编码器信息:单圈多圈位置、分辨率_请求
                    if (!Task_Status)//开启多线程
                    {
                        CurrentTask = Task.Run(() =>
                        {
                            Communication.Communication_Function.Requst_Processing();
                        });
                    }
                    else//若线程已开启则继续线程
                    {
                        ResetEvent.Set();
                    }
                    btn_Connect.Text = "断开";//改变按钮显示
                    btn_Scan.Enabled = false;
                    cbx_PortName.Enabled = false;
                    cbx_BaudRate.Enabled = false;
                }
                else
                {
                    MessageBox.Show("串口打开失败","提示");
                }
            }
            else if (btn_Connect.Text == "断开")
            {
                SerialPort_Function.Close_SerialPort();//串口关闭
                if (CurrentTask != null)//暂停线程
                {
                    if (CurrentTask.Status == TaskStatus.Running)
                    {
                       Task_Status = ResetEvent.Reset();
                    }
                }
                btn_Connect.Text = "连接";//改变按钮显示
                btn_Scan.Enabled = true;
                cbx_PortName.Enabled = true;
                cbx_BaudRate.Enabled = true;
                Communication.ZeroErr_Communication.Struct_Request_Mode_Init();//清除所有通讯请求
            }
        }

        private void Print_Message(string text)//更新信息委托
        {
            if (rtbx_Message.InvokeRequired)
            {
                Action<string> a = Print_Message;
                this.Invoke(a, new object[] { text });
            }
            else
            {
                rtbx_Message.AppendText(DateTime.Now + "  " + text);
            }
        }

        private void Close_Port()//热拔插动作委托
        {
            Communication.SerialPort_Function.Close_SerialPort();//关闭串口
            Communication.ZeroErr_Communication.Struct_Request_Mode_Init();//清除所有通讯请求
            ResetEvent.Reset();//暂停线程扫描
        }

        private void btn_Set_Single_To_Zero_Click(object sender, EventArgs e)//单圈清零按钮点击事件
        {
            if (Communication.SerialPort_Function.eCoder_SerialPort.IsOpen)
            {
                Communication.ZeroErr_Communication.Struct_Request_Mode.set_Single_Turn_To_Zero = true;
            }
        }

        private void btn_Set_Multi_To_Zero_Click(object sender, EventArgs e)//多圈清零按钮点击事件
        {
            if (Communication.SerialPort_Function.eCoder_SerialPort.IsOpen)
            {
                Communication.ZeroErr_Communication.Struct_Request_Mode.set_Multi_Turn_To_Zero = true;
            }
        }
        
        private void btn_Read_Status_Click(object sender, EventArgs e)//读取状态按钮点击事件
        {
            if (Communication.SerialPort_Function.eCoder_SerialPort.IsOpen)
            {
                Communication.ZeroErr_Communication.Struct_Request_Mode.read_eCoder_Status = true;
            }
        }

        private void btn_Clear_Message_Click(object sender, EventArgs e)//清除按钮点击事件
        {
            if (rtbx_Message.Text != string.Empty)
            {
                rtbx_Message.Clear();
            }
        }

        private void checkbx_Only_Show_Single_CheckedChanged(object sender, EventArgs e)//按钮切换事件
        {
            if (checkbx_Only_Show_Single.Checked)
            {
                Communication.Data_Conversion.eCoder_Angle_Mode_Change = true;
            }
            else
            {
                Communication.Data_Conversion.eCoder_Angle_Mode_Change = false;
            }
        }
    }
}
